<?php

/**
 * Входящий вебхук Bitrix
 */
const BITRIX = 'Шаг_1';
/**
 * Уровень доступа Memberlux
 */
const LEVEL = Шаг_2;
/**
 * Название воронки продаж
 */
const CATEGORY = 'Шаг_3';
/**
 * Название этапа воронки
 */
const STAGE = "Шаг_4";

$post = json_decode(file_get_contents('php://input'), true);
$TITLE = "Memberlux {$post['email']}";

if ((int)$post['key']['term_id'] !== LEVEL) {
    die;
}

if (empty($_GET['handler'])) {
    no_blocking($post);
    die;
}

$contact_data = [
    "NAME" => $post['first_name'],
    "SECOND_NAME" => $post['patronymic'],
    "LAST_NAME" => $post['last_name'],
    "EMAIL" => $post['email'],
    "PHONE" => $post['phone'],
];

try {
    $category_id = get_category(CATEGORY);
    $stage_id = get_stage(STAGE, $category_id);
    $contact_id = create_contact($contact_data);

    $deal_data = create_deal([
        "TITLE" => $TITLE,
        "STAGE_ID" => $stage_id,
        "CATEGORY_ID" => $category_id,
        "CONTACT_ID" => $contact_id,
    ]);

    file_put_contents(
        __DIR__ . '/webhook-success-log.txt',
        date('H:i:s d.m.Y') . ' ' . var_export($deal_data, true) . PHP_EOL,
        FILE_APPEND
    );
} catch (Exception $exception) {
    file_put_contents(
        __DIR__ . '/webhook-error-log.txt',
        date('H:i:s d.m.Y') . ' ' . var_export($exception->getMessage(), true) . PHP_EOL,
        FILE_APPEND
    );
}

function no_blocking($data)
{
    $is_https = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off');
    $current_url = ($is_https ? 'https://' : 'http://') . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
    $curl = curl_init();
    curl_setopt_array($curl, [
        CURLOPT_URL => $current_url . '?handler=1',
        CURLOPT_RETURNTRANSFER => 1,
        CURLOPT_POST => 1,
        CURLOPT_POSTFIELDS => json_encode($data),
        CURLOPT_HTTPHEADER => [
            'Content-Type: application/json'
        ],
        CURLOPT_TIMEOUT => 1,
    ]);
    curl_exec($curl);
    curl_close($curl);
}

/**
 * @param $params array{
 *     EMAIL: string,
 *     NAME: string,
 *     SECOND_NAME: string,
 *     LAST_NAME: string,
 *     PHONE: string
 * }
 * @return int
 * @throws Exception
 */
function create_contact(array $params): int
{
    if (empty($params['EMAIL'])) {
        throw new Exception('Field EMAIL is required');
    }

    $search = bx_request([
        "route" => "crm.contact.list",
        "method" => "POST",
        "body" => ['filter' => ["EMAIL" => $params['EMAIL']],]
    ]);

    if (!empty($search['result'])) {
        $existing_contact = $search['result'][0];
        $id = $existing_contact['ID'];
    } else {
        $fields = $params;
        $fields['EMAIL'] = [["VALUE" => $params['EMAIL']]];
        if (!empty($params['PHONE'])) {
            $fields['PHONE'] = [["VALUE" => $params['PHONE']]];
        }
        $response = bx_request([
            "route" => "crm.contact.add",
            "method" => "POST",
            "body" => ["fields" => $fields]
        ]);
        $id = $response['result'];
    }

    if (empty($id)) {
        throw new Exception('Failed to create contact ' . $params['email']);
    }

    return $id;
}

function create_deal($params)
{
    return bx_request([
        "route" => "crm.deal.add",
        "method" => "POST",
        "body" => ["fields" => $params]
    ]);
}

/**
 * @throws Exception
 */
function get_category(string $name): int
{
    $response = bx_request([
        'route' => 'crm.category.list',
        'method' => 'POST',
        'body' => ['entityTypeId' => 2]
    ]);

    if (empty($response['result']) || empty($response['result']['categories'])) {
        throw new Exception('Bx request crm.category.list error');
    }

    $categories = $response['result']['categories'];
    $search = array_search($name, array_column($categories, 'name'));

    if ($search === false) {
        throw new Exception("Category '$name' not found");
    }

    return $categories[$search]['id'];
}

/**
 * @throws Exception
 */
function get_stage(string $name, int $category_id): string {
    $ENTITY_ID = 'DEAL_STAGE'. ($category_id === 0 ? '' : "_$category_id");
    $response = bx_request([
        'route' => 'crm.status.list',
        'method' => 'POST',
        'body' => ['filter' => ['ENTITY_ID' => $ENTITY_ID]]
    ]);

    if (empty($response['result'])) {
        throw new Exception('Bx request crm.status.list error');
    }

    $stages = $response['result'];

    $search = array_search($name, array_column($stages, 'NAME'));

    if ($search === false) {
        throw new Exception("Stage '$name' in category $category_id not found");
    }

    return $stages[$search]['STATUS_ID'];
}

/**
 * @throws Exception
 */
function bx_request($params)
{
    $obCurl = curl_init();
    $base_url = BITRIX . $params['route'];
    curl_setopt($obCurl, CURLOPT_URL, $base_url);
    curl_setopt($obCurl, CURLOPT_RETURNTRANSFER, 1);

    if (!empty($params['query'])) {
        $queryParams = http_build_query($params['query']);
        curl_setopt($obCurl, CURLOPT_URL, "{$base_url}?$queryParams");
    }

    if ($params['method'] = 'POST') {
        curl_setopt($obCurl, CURLOPT_POST, true);
        if (!empty($params['body'])) {
            $postFields = http_build_query($params['body']);
            curl_setopt($obCurl, CURLOPT_POSTFIELDS, $postFields);
        }
    }

    $data = curl_exec($obCurl);
    $error = curl_errno($obCurl) ? curl_error($obCurl) : null;

    curl_close($obCurl);

    if ($error) {
        throw new Exception($error);
    }

    return json_decode($data, true);
}